@@ -1,4 +1,4 @@
-#' A data cube tbl.
+#' A data cube tbl
 #'
 #' A cube tbl stores data in a compact array format where dimension
 #' names are not needlessly repeated. They are particularly appropriate for
@@ -7,7 +7,7 @@
 #' Compared to data frames, they will occupy much less memory when variables
 #' are crossed, not nested.
 #'
-#' \code{tbl_cube} support is currently experimental and little performance
+#' `tbl_cube` support is currently experimental and little performance
 #' optimisation has been done, but you may find them useful if your data
 #' already comes in this form, or you struggle with the memory overhead of the
 #' sparse/crossed of data frames.  There is no support for hierarchical
@@ -19,21 +19,21 @@
 #' Manipulation functions:
 #'
 #' \itemize{
-#'   \item \code{select} (M)
+#'   \item `select()` (M)
 #'
-#'   \item \code{summarise} (M), corresponds to roll-up, but rather more
+#'   \item `summarise()` (M), corresponds to roll-up, but rather more
 #'     limited since there are no hierarchies.
 #'
-#'   \item \code{filter} (D), corresponds to slice/dice.
+#'   \item `filter()` (D), corresponds to slice/dice.
 #'
-#'   \item \code{mutate} (M) is not implemented, but should be relatively
-#'   straightforward given the implementation of \code{summarise}.
+#'   \item `mutate()` (M) is not implemented, but should be relatively
+#'   straightforward given the implementation of `summarise`.
 #'
-#'   \item \code{arrange} (D?) Not implemented: not obvious how much sense
+#'   \item `arrange()` (D?) Not implemented: not obvious how much sense
 #'     it would make
 #' }
 #'
-#' Joins: not implemented. See \code{vignettes/joins.graffle} for ideas.
+#' Joins: not implemented. See `vignettes/joins.graffle` for ideas.
 #' Probably straightforward if you get the indexes right, and that's probably
 #' some straightforward array/tensor operation.
 #'
@@ -41,7 +41,7 @@
 #' @param dimensions A named list of vectors. A dimension is a variable
 #'   whose values are known before the experiement is conducted; they are
 #'   fixed by design (in \pkg{reshape2} they are known as id variables).
-#'   \code{tbl_cubes} are dense which means that almost every combination of
+#'   `tbl_cubes` are dense which means that almost every combination of
 #'   the dimensions should have associated measurements: missing values require
 #'   an explicit NA, so if the variables are nested, not crossed, the
 #'   majority of the data structure will be empty. Dimensions are typically,
@@ -50,8 +50,8 @@
 #'   actually measured, and is not known in advance. The dimension of each
 #'   array should be the same as the length of the dimensions. Measures are
 #'   typically, but not always, continuous values.
-#' @seealso \code{\link{as.tbl_cube}} for ways of coercing existing data
-#'   structures into a \code{tbl_cube}.
+#' @seealso [as.tbl_cube()] for ways of coercing existing data
+#'   structures into a `tbl_cube`.
 #' @examples
 #' # The built in nasa dataset records meterological data (temperature,
 #' # cloud cover, ozone etc) for a 4d spatio-temporal dataset (lat, long,
@@ -86,22 +86,29 @@
 tbl_cube <- function(dimensions, measures) {
   if (!is.list(dimensions) || any_apply(dimensions, Negate(is.atomic)) ||
       is.null(names(dimensions))) {
-    stop("Dimensions must be a named list of vectors", call. = FALSE)
+    bad_args("dimensions", "must be a named list of vectors, ",
+      "not {type_of(dimensions)}"
+    )
   }
 
   if (!is.list(measures) || any_apply(measures, Negate(is.array)) ||
     is.null(names(measures))) {
-    stop("Measures must be a named list of arrays", call. = FALSE)
+    bad_args("measures", "must be a named list of arrays, ",
+      "not {type_of(measures)}"
+    )
   }
 
   # Check measures have correct dimensions
   dims <- vapply(dimensions, length, integer(1), USE.NAMES = FALSE)
-  dims_ok <- vapply(measures, function(x) identical(unname(dim(x)), dims),
-    logical(1))
+  dims_ok <- vapply(
+    measures, function(x) identical(unname(dim(x)), dims),
+    logical(1)
+  )
   if (any(!dims_ok)) {
     bad <- names(measures)[!dims_ok]
-    stop("Measures ", paste0(bad, collapse = ", "), " don't have correct ",
-      "dimensions (", paste0(dims, collapse = " x "), ")", call. = FALSE)
+    bad_measures(bad, "needs dimensions {fmt_dims(dims)}, not {bad_dim}",
+      bad_dim = fmt_dims(dim(measures[!dims_ok][[1L]]))
+    )
   }
 
   structure(list(dims = dimensions, mets = measures), class = "tbl_cube")
@@ -122,11 +129,12 @@ same_src.tbl_cube <- function(x, y) {
 
 #' @export
 print.tbl_cube <- function(x, ...) {
-  cat("Source: local array ", dim_desc(x), "\n",
-    sep = "")
+  cat("Source: local array ", dim_desc(x), "\n", sep = "")
   if (!is.null(x$groups)) {
-    cat("Grouped by: ", paste(names(x$dims)[x$groups], collapse = ", "),
-      "\n", sep = "")
+    cat(
+      "Grouped by: ", paste(names(x$dims)[x$groups], collapse = ", "), "\n",
+      sep = ""
+    )
   }
 
   # Dimensions
@@ -144,11 +152,11 @@ print.tbl_cube <- function(x, ...) {
 
 # Coercion methods (from tbl_cube) ---------------------------------------------
 
-#' Coerce a \code{tbl_cube} to other data structures
+#' Coerce a `tbl_cube` to other data structures
 #'
 #' Supports conversion to tables, data frames, tibbles.
 #'
-#' @param x a \code{tbl_cube}
+#' @param x a `tbl_cube`
 #' @param ... Passed on to individual methods; otherwise ignored.
 #' @param measure A measure name or index, default: the first measure
 #' @name as.table.tbl_cube
@@ -176,7 +184,7 @@ as.data.frame.tbl_cube <- function(x, ...) {
 
 #' @rdname as.table.tbl_cube
 #' @description For a cube, the data frame returned by
-#'   \code{\link[tibble]{as_data_frame}} resulting data frame contains the
+#'   [tibble::as_data_frame()] resulting data frame contains the
 #'   dimensions as character values (and not as factors).
 #' @export
 as_data_frame.tbl_cube <- function(x, ...) {
@@ -185,7 +193,7 @@ as_data_frame.tbl_cube <- function(x, ...) {
 
 # Coercion methods -------------------------------------------------------------
 
-#' Coerce an existing data structure into a \code{tbl_cube}
+#' Coerce an existing data structure into a `tbl_cube`
 #'
 #' @param x an object to convert. Built in methods will convert arrays,
 #'   tables and data frames.
@@ -197,8 +205,9 @@ as.tbl_cube <- function(x, ...) UseMethod("as.tbl_cube")
 #' @rdname as.tbl_cube
 #' @param dim_names names of the dimesions. Defaults to the names of
 #' @param met_name a string to use as the name for the measure
-#'   the \code{\link{dimnames}}.
-as.tbl_cube.array <- function(x, dim_names = names(dimnames(x)), met_name = deparse(substitute(x)), ...) {
+#'   the [dimnames()].
+as.tbl_cube.array <- function(x, dim_names = names(dimnames(x)), met_name = deparse(substitute(x)),
+                              ...) {
   force(met_name)
 
   dims <- dimnames(x)
@@ -216,7 +225,8 @@ undimname <- function(x) {
 
 #' @export
 #' @rdname as.tbl_cube
-as.tbl_cube.table <- function(x, dim_names = names(dimnames(x)), met_name = "Freq", ...) {
+as.tbl_cube.table <- function(x, dim_names = names(dimnames(x)), met_name = "Freq",
+                              ...) {
   as.tbl_cube.array(unclass(x), dim_names = dim_names, met_name = met_name)
 }
 
@@ -232,13 +242,14 @@ guess_met <- function(df) {
     met <- names(df)[is_num]
   }
 
-  message("Using ", paste(met, collapse = ", "), " as measure column(s): use met_name to override.")
+  inform(paste0("Using ", paste(met, collapse = ", "), " as measure column(s): use `met_name` to override."))
   met
 }
 
 #' @export
 #' @rdname as.tbl_cube
-as.tbl_cube.data.frame <- function(x, dim_names = NULL, met_name = guess_met(x), ...) {
+as.tbl_cube.data.frame <- function(x, dim_names = NULL, met_name = guess_met(x),
+                                   ...) {
   if (is.null(dim_names)) {
     dim_names <- setdiff(names(x), met_name)
   } else {
@@ -254,10 +265,6 @@ as.tbl_cube.data.frame <- function(x, dim_names = NULL, met_name = guess_met(x),
     met_name <- names(x)[met_name]
   }
 
-  if (is.null(dim_names) && is.null(met_name)) {
-    stop("At least one of dim_names and met_name must be non-NULL.", call. = FALSE)
-  }
-
   dims <- lapply(x[dim_names], unique)
   n <- vapply(dims, length, integer(1))
 
@@ -267,8 +274,9 @@ as.tbl_cube.data.frame <- function(x, dim_names = NULL, met_name = guess_met(x),
     dupe_row <- anyDuplicated(all[dim_names])
     dupe <- unlist(all[dupe_row, dim_names])
 
-    stop("Duplicate combination of dimension variables: ",
-         paste(names(dupe), "=", dupe, collapse = ", "), call. = FALSE)
+    bad_args("x", "must be unique in all combinations of dimension variables, ",
+      'duplicates: {fmt_named(dupe)}'
+    )
   }
 
   mets <- lapply(met_name, function(i) array(all[[i]], unname(n)))
@@ -281,32 +289,40 @@ as.tbl_cube.data.frame <- function(x, dim_names = NULL, met_name = guess_met(x),
 # Verbs -------------------------------------------------------------------
 
 #' @export
-select_.tbl_cube <- function(.data, ..., .dots) {
-  dots <- lazyeval::all_dots(.dots, ...)
-  vars <- select_vars_(names(.data$mets), dots)
-
+select.tbl_cube <- function(.data, ...) {
+  vars <- select_vars(names(.data$mets), ...)
   .data$mets <- .data$mets[vars]
   .data
 }
+#' @export
+select_.tbl_cube <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  select(.data, !!! dots)
+}
 
 #' @export
-rename_.tbl_cube <- function(.data, ..., .dots) {
-  dots <- lazyeval::all_dots(.dots, ...)
-  vars <- rename_vars_(names(.data$mets), dots)
-
+rename.tbl_cube <- function(.data, ...) {
+  vars <- rename_vars(names(.data$mets), ...)
   .data$mets <- .data$mets[vars]
   .data
 }
+#' @export
+rename_.tbl_cube <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  rename(.data, !!! dots)
+}
 
 
 #' @export
-filter_.tbl_cube <- function(.data, ..., .dots) {
-  dots <- lazyeval::all_dots(.dots, ...)
+filter.tbl_cube <- function(.data, ...) {
+  dots <- quos(...)
 
-  idx <- vapply(dots, function(d) find_index_check(d$expr, names(.data$dims)),
-    integer(1))
-  for(i in seq_along(dots)) {
-    sel <- eval(dots[[i]]$expr, .data$dims, dots[[i]]$env)
+  idx <- map2_int(
+    seq_along(dots), dots,
+    function(i, d) find_index_check(i, d, names(.data$dims))
+  )
+  for (i in seq_along(dots)) {
+    sel <- eval_tidy(dots[[i]], .data$dims)
     sel <- sel & !is.na(sel)
 
     .data$dims[[idx[i]]] <- .data$dims[[idx[i]]][sel]
@@ -315,11 +331,18 @@ filter_.tbl_cube <- function(.data, ..., .dots) {
 
   .data
 }
+#' @export
+filter_.tbl_cube <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  filter(.data, !!! dots)
+}
 
-find_index_check <- function(x, names) {
-  idx <- find_index(x, names)
+find_index_check <- function(i, x, names) {
+  idx <- find_index(f_rhs(x), names)
   if (length(idx) != 1) {
-    stop(deparse(x), " does not refer to exactly one dimension.", call. = FALSE)
+    bad_calls(x, "must refer to exactly one dimension, ",
+      "not {fmt_obj(names[idx])}"
+    )
   }
   idx
 }
@@ -338,20 +361,27 @@ find_index <- function(x, names) {
 }
 
 #' @export
-group_by_.tbl_cube <- function(.data, ..., .dots, add = FALSE) {
-  groups <- group_by_prepare(.data, ..., .dots = .dots, add = add)
+group_by.tbl_cube <- function(.data, ..., add = FALSE) {
+  groups <- group_by_prepare(.data, ..., add = add)
 
   # Convert symbols to indices
-  nms <- names(groups$data$dims)
-  nms_list <- as.list(setNames(seq_along(nms), nms))
-
-  groups$data$groups <- unlist(lapply(groups$groups, eval, nms_list))
+  groups$data$groups <- match(groups$group_names, names(groups$data$dims))
   groups$data
 }
+#' @export
+group_by_.tbl_cube <- function(.data, ..., .dots = list(), add = FALSE) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  group_by(.data, !!! dots, add = add)
+}
 
 #' @export
 groups.tbl_cube <- function(x) {
-  lapply(x$dims, as.name)[x$group]
+  lapply(group_vars(x), as.name)
+}
+
+#' @export
+group_vars.tbl_cube <- function(x) {
+  names(x$dims[x$groups])
 }
 
 # mutate and summarise operate similarly need to evaluate variables in special
@@ -359,34 +389,41 @@ groups.tbl_cube <- function(x) {
 # for better performance
 
 #' @export
-summarise_.tbl_cube <- function(.data, ..., .dots) {
-  dots <- lazyeval::all_dots(.dots, ..., all_named = TRUE)
+summarise.tbl_cube <- function(.data, ...) {
+  dots <- named_quos(...)
 
   out_dims <- .data$dims[.data$groups]
-  n <- vapply(out_dims, length, integer(1))
+  n <- map_int(out_dims, length)
 
   out_mets <- list()
   for (nm in names(dots)) {
     out_mets[[nm]] <- array(logical(), n)
   }
 
-  slices <- expand.grid(lapply(out_dims, seq_along), KEEP.OUT.ATTRS = FALSE)
+  slices <- expand.grid(map(out_dims, seq_along), KEEP.OUT.ATTRS = FALSE)
 
   # Loop over each group
   for (i in seq_len(nrow(slices))) {
-    index <- as.list(slices[i, , drop = FALSE])
-    mets <- lapply(.data$mets, subs_index, i = .data$groups, val = index,
-      drop = TRUE)
+    index <- as_list(slices[i, , drop = FALSE])
+    mets <- map(
+      .data$mets, subs_index, i = .data$groups, val = index,
+      drop = TRUE
+    )
 
     # Loop over each expression
     for (j in seq_along(dots)) {
-      res <- eval(dots[[j]]$expr, mets, dots[[j]]$env)
+      res <- eval_tidy(dots[[j]], mets)
       out_mets[[j]][i] <- res
     }
   }
 
   structure(list(dims = out_dims, mets = out_mets), class = "tbl_cube")
 }
+#' @export
+summarise_.tbl_cube <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  summarise(.data, !!! dots)
+}
 
 subs_index <- function(x, i, val, drop = FALSE) {
   dims <- length(dim(x) %||% 1)
@@ -396,20 +433,23 @@ subs_index <- function(x, i, val, drop = FALSE) {
   if (length(i) == 1 && is.atomic(val)) {
     args[[i]] <- quote(val)
   } else if (length(i) >= 1 && is.list(val)) {
-    exprs <- lapply(seq_along(i), function(i) as.call(c(quote(`[[`), quote(val), i)))
+    exprs <- lapply(
+      seq_along(i),
+      function(i) as.call(c(quote(`[[`), quote(val), i))
+    )
     args[i] <- exprs
   } else {
-    stop("Invalid input", call. = FALSE)
+    abort("Invalid input")
   }
 
   args$drop <- drop
 
   call <- as.call(c(quote(`[`), quote(x), args))
-  eval(call)
+  eval_bare(call)
 }
 
 
 #' @export
 auto_copy.tbl_cube <- function(x, y, copy = FALSE, ...) {
-  stop("Copying not supported by tbl_cube", call. = FALSE)
+  abort("Copying not supported by tbl_cube")
 }
